@@ -1,4 +1,6 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim: set sw=4 sts=4 ts=4 expandtab: */
+/*
 
    rsvg-convert.c: Command line utility for exercising rsvg with cairo.
  
@@ -50,6 +52,10 @@
 #include <cairo-svg.h>
 #endif
 
+#ifdef CAIRO_HAS_XML_SURFACE
+#include <cairo-xml.h>
+#endif
+
 static void
 display_error (GError * err)
 {
@@ -74,17 +80,17 @@ rsvg_handle_new_from_stdio_file (FILE * f, GError ** error)
 
         if (length > 0) {
             if (!rsvg_handle_write (handle, buffer, length, error)) {
-                g_object_unref (G_OBJECT (handle));
+                g_object_unref (handle);
                 return NULL;
             }
         } else if (ferror (f)) {
-            g_object_unref (G_OBJECT (handle));
+            g_object_unref (handle);
             return NULL;
         }
     }
 
     if (!rsvg_handle_close (handle, error)) {
-        g_object_unref (G_OBJECT (handle));
+        g_object_unref (handle);
         return NULL;
     }
 
@@ -109,8 +115,8 @@ static cairo_status_t
 rsvg_cairo_write_func (void *closure, const unsigned char *data, unsigned int length)
 {
     if (fwrite (data, 1, length, (FILE *) closure) == length)
-		return CAIRO_STATUS_SUCCESS;
-	return CAIRO_STATUS_WRITE_ERROR;
+        return CAIRO_STATUS_SUCCESS;
+    return CAIRO_STATUS_WRITE_ERROR;
 }
 
 int
@@ -128,7 +134,7 @@ main (int argc, char **argv)
     char *format = NULL;
     char *output = NULL;
     int keep_aspect_ratio = FALSE;
-    guint32 background_color = NULL;
+    guint32 background_color = 0;
     char *background_color_str = NULL;
     char *base_uri = NULL;
     gboolean using_stdin = FALSE;
@@ -159,7 +165,7 @@ main (int argc, char **argv)
         {"height", 'h', 0, G_OPTION_ARG_INT, &height,
          N_("height [optional; defaults to the SVG's height]"), N_("<int>")},
         {"format", 'f', 0, G_OPTION_ARG_STRING, &format,
-         N_("save format [optional; defaults to 'png']"), N_("[png, pdf, ps, svg]")},
+         N_("save format [optional; defaults to 'png']"), N_("[png, pdf, ps, svg, xml, recording]")},
         {"output", 'o', 0, G_OPTION_ARG_STRING, &output,
          N_("output filename [optional; defaults to stdout]"), NULL},
         {"keep-aspect-ratio", 'a', 0, G_OPTION_ARG_NONE, &keep_aspect_ratio,
@@ -172,10 +178,10 @@ main (int argc, char **argv)
         {NULL}
     };
 
-	/* Set the locale so that UTF-8 filenames work */
-	setlocale(LC_ALL, "");
+    /* Set the locale so that UTF-8 filenames work */
+    setlocale(LC_ALL, "");
 
-	g_thread_init(NULL);
+    g_thread_init(NULL);
 
     g_option_context = g_option_context_new (_("- SVG Converter"));
     g_option_context_add_main_entries (g_option_context, options_table, NULL);
@@ -292,6 +298,18 @@ main (int argc, char **argv)
             else if (!strcmp (format, "svg"))
                 surface = cairo_svg_surface_create_for_stream (rsvg_cairo_write_func, output_file,
                                                                dimensions.width, dimensions.height);
+#endif
+#ifdef CAIRO_HAS_XML_SURFACE
+            else if (!strcmp (format, "xml")) {
+                cairo_device_t *device = cairo_xml_create_for_stream (rsvg_cairo_write_func, output_file);
+                surface = cairo_xml_surface_create (device, CAIRO_CONTENT_COLOR_ALPHA,
+                                                    dimensions.width, dimensions.height);
+                cairo_device_destroy (device);
+            }
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE (1, 10, 0)
+            else if (!strcmp (format, "recording"))
+                surface = cairo_recording_surface_create (CAIRO_CONTENT_COLOR_ALPHA, NULL);
+#endif
 #endif
             else {
                 fprintf (stderr, _("Unknown output format."));
@@ -318,13 +336,25 @@ main (int argc, char **argv)
 
         if (!format || !strcmp (format, "png"))
             cairo_surface_write_to_png_stream (surface, rsvg_cairo_write_func, output_file);
-        else
+#if CAIRO_HAS_XML_SURFACE && CAIRO_VERSION >= CAIRO_VERSION_ENCODE (1, 10, 0)
+        else if (!strcmp (format, "recording")) {
+            cairo_device_t *device = cairo_xml_create_for_stream (rsvg_cairo_write_func, output_file);
+            cairo_xml_for_recording_surface (device, surface);
+            cairo_device_destroy (device);
+        }
+#endif
+        else if (!strcmp (format, "xml"))
+          ;
+        else if (!strcmp (format, "svg") || !strcmp (format, "pdf") || !strcmp (format, "ps"))
             cairo_show_page (cr);
+        else
+          g_assert_not_reached ();
 
-        g_object_unref (G_OBJECT (rsvg));
+        g_object_unref (rsvg);
     }
 
     cairo_destroy (cr);
+
     cairo_surface_destroy (surface);
 
     fclose (output_file);
